@@ -9,7 +9,7 @@
 
 // Qt includes.
 #include <QAction>
-#include <QGLFormat>
+#include <QActionGroup>
 #include <QHBoxLayout>
 #include <QLineEdit>
 #include <QStatusBar>
@@ -19,16 +19,13 @@
 #include <KActionCollection>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KStandardGameAction>
+#include <KGameStandardAction>
 #include <KToggleAction>
 
 // Local includes.
 #include "game.h"
 #include "gameglview.h"
 
-// Shorthand for reference to actions.
-#define ACTION(x)   (actionCollection()->action(x))
-
 Kubrick::Kubrick () :
     singmasterMoves (nullptr)
 {
@@ -62,9 +59,9 @@ Kubrick::Kubrick () :
     auto slot = &Kubrick::patternSelected;
     Q_UNUSED(slot)
     fillDemoList (patterns, patternList,
-			"patterns_list", &Kubrick::patternSelected);
+			QStringLiteral("patterns_list"), &Kubrick::patternSelected);
     fillDemoList (solvingMoves, movesList,
-			"demo_moves_list", &Kubrick::movesSelected);
+			QStringLiteral("demo_moves_list"), &Kubrick::movesSelected);
 
     // Set up a status bar.
     statusBar()->show ();
@@ -144,20 +141,20 @@ const KLazyLocalizedString Kubrick::patternMovesInfo = kli18n(
 
 
 const Kubrick::DemoItem Kubrick::patterns [] = {
-    {"",			kli18n("Info")},
-    {"p333X6.kbk",		kli18n("3x3x3, 6 X")},
-    {"p333X2.kbk",		kli18n("3x3x3, 2 X")},
-    {"p333Spot6.kbk",		kli18n("3x3x3, 6 Spot")},
-    {"p333Spot4.kbk",		kli18n("3x3x3, 4 Spot")},
-    {"p333Plus4.kbk",		kli18n("3x3x3, 4 Plus")},
-    {"p333Bar4.kbk",		kli18n("3x3x3, 4 Bar")},
-    {"p333U6.kbk",		kli18n("3x3x3, 6 U")},
-    {"p333U4.kbk",		kli18n("3x3x3, 4 U")},
-    {"p333Snake.kbk",		kli18n("3x3x3, Snake")},
-    {"p333Worm.kbk",		kli18n("3x3x3, Worm")},
-    {"p333Tricolor6.kbk",	kli18n("3x3x3, Tricolor")},
-    {"p333DoubleCube.kbk",	kli18n("3x3x3, Double Cube")},
-    {"END",			KLazyLocalizedString()}
+    {QString(),                         	kli18n("Info")},
+    {QStringLiteral("p333X6.kbk"),		kli18n("3x3x3, 6 X")},
+    {QStringLiteral("p333X2.kbk"),		kli18n("3x3x3, 2 X")},
+    {QStringLiteral("p333Spot6.kbk"),		kli18n("3x3x3, 6 Spot")},
+    {QStringLiteral("p333Spot4.kbk"),		kli18n("3x3x3, 4 Spot")},
+    {QStringLiteral("p333Plus4.kbk"),		kli18n("3x3x3, 4 Plus")},
+    {QStringLiteral("p333Bar4.kbk"),		kli18n("3x3x3, 4 Bar")},
+    {QStringLiteral("p333U6.kbk"),		kli18n("3x3x3, 6 U")},
+    {QStringLiteral("p333U4.kbk"),		kli18n("3x3x3, 4 U")},
+    {QStringLiteral("p333Snake.kbk"),		kli18n("3x3x3, Snake")},
+    {QStringLiteral("p333Worm.kbk"),		kli18n("3x3x3, Worm")},
+    {QStringLiteral("p333Tricolor6.kbk"),	kli18n("3x3x3, Tricolor")},
+    {QStringLiteral("p333DoubleCube.kbk"),	kli18n("3x3x3, Double Cube")},
+    {QStringLiteral("END"),			KLazyLocalizedString()}
 };
 
 
@@ -179,19 +176,19 @@ const KLazyLocalizedString Kubrick::solvingMovesInfo = kli18n(
 
 
 const Kubrick::DemoItem Kubrick::solvingMoves [] = {
-    {"",		    kli18n("Info")},
-    {"m333Layer1.kbk",	    kli18n("3x3x3 Layer 1, Edges First")},
-    {"m333MEdge1.kbk",	    kli18n("3x3x3 Layer 2, Edge from Bottom Right")},
-    {"m333MEdge2.kbk",	    kli18n("3x3x3 Layer 2, Edge from Bottom Left")},
-    {"m333LLEdgeFlip.kbk",  kli18n("3x3x3 Layer 3, Flip Edge Pieces")},
-    {"m333LLCornerPos.kbk", kli18n("3x3x3 Layer 3, Place Corners")},
-    {"m333LLCornerRot.kbk", kli18n("3x3x3 Layer 3, Twist Corners")},
-    {"m333LLEdgePos.kbk",   kli18n("3x3x3 Layer 3, Place Edges and DONE!")},
-    {"m333Complete.kbk",    kli18n("3x3x3 Cube, Complete Solution")},
-    {"m333E2prX.kbk",	    kli18n("3x3x3 Swap 2 Pairs of Edges")},
-    {"m333CTwirl2.kbk",	    kli18n("3x3x3 Untwist 2 Corners")},
-    {"m333EFlip2.kbk",	    kli18n("3x3x3 Flip 2 Edges")},
-    {"END",		    KLazyLocalizedString()}
+    {QString(),	                	    kli18n("Info")},
+    {QStringLiteral("m333Layer1.kbk"),	    kli18n("3x3x3 Layer 1, Edges First")},
+    {QStringLiteral("m333MEdge1.kbk"),	    kli18n("3x3x3 Layer 2, Edge from Bottom Right")},
+    {QStringLiteral("m333MEdge2.kbk"),	    kli18n("3x3x3 Layer 2, Edge from Bottom Left")},
+    {QStringLiteral("m333LLEdgeFlip.kbk"),  kli18n("3x3x3 Layer 3, Flip Edge Pieces")},
+    {QStringLiteral("m333LLCornerPos.kbk"), kli18n("3x3x3 Layer 3, Place Corners")},
+    {QStringLiteral("m333LLCornerRot.kbk"), kli18n("3x3x3 Layer 3, Twist Corners")},
+    {QStringLiteral("m333LLEdgePos.kbk"),   kli18n("3x3x3 Layer 3, Place Edges and DONE!")},
+    {QStringLiteral("m333Complete.kbk"),    kli18n("3x3x3 Cube, Complete Solution")},
+    {QStringLiteral("m333E2prX.kbk"),	    kli18n("3x3x3 Swap 2 Pairs of Edges")},
+    {QStringLiteral("m333CTwirl2.kbk"),	    kli18n("3x3x3 Untwist 2 Corners")},
+    {QStringLiteral("m333EFlip2.kbk"),	    kli18n("3x3x3 Flip 2 Edges")},
+    {QStringLiteral("END"),		    KLazyLocalizedString()}
 };
 
 
@@ -200,8 +197,8 @@ const Kubrick::DemoItem Kubrick::solvingMoves [] = {
 void Kubrick::initGUI()
 {
     // Game menu.
-    QAction * newAction =	KStandardGameAction::gameNew (
-				game, SLOT (newPuzzle()), this);
+    QAction * newAction =	KGameStandardAction::gameNew (
+				game, &Game::newPuzzle, this);
     actionCollection()->addAction (newAction->objectName(), newAction);
     newAction->setText		(i18n("&New Puzzle"));
     newAction->setToolTip	(i18n("Start a new puzzle."));
@@ -210,8 +207,8 @@ void Kubrick::initGUI()
 				"dimensions and number of shuffling moves."));
 
     QAction *
-    a =				KStandardGameAction::load (
-				game, SLOT (load()), this);
+    a =				KGameStandardAction::load (
+				game, &Game::load, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setText			(i18n("&Load Puzzle..."));
     a->setToolTip		(i18n("Reload a saved puzzle from a file."));
@@ -220,8 +217,8 @@ void Kubrick::initGUI()
 				"settings, current state and history of "
 				"moves."));
 
-    a =				KStandardGameAction::save (
-				game, SLOT (save()), this);
+    a =				KGameStandardAction::save (
+				game, &Game::save, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setText			(i18n("&Save Puzzle..."));
     a->setToolTip		(i18n("Save the puzzle on a file."));
@@ -229,13 +226,13 @@ void Kubrick::initGUI()
 				"its dimensions, settings, current state and "
 				"history of moves."));
 
-    a =				KStandardGameAction::saveAs (
-				game, SLOT (saveAs()), this);
+    a =				KGameStandardAction::saveAs (
+				game, &Game::saveAs, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setText			(i18n("&Save Puzzle As..."));
 
-    a =				KStandardGameAction::
-				restart (game, SLOT (undoAll()), this);
+    a =				KGameStandardAction::
+				restart (game, &Game::undoAll, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setText			(i18n("Restart &Puzzle..."));
     a->setToolTip		(i18n("Undo all previous moves and start "
@@ -243,34 +240,34 @@ void Kubrick::initGUI()
     a->setWhatsThis		(i18n("Undo all previous moves and start "
 				"again."));
 
-    a =				KStandardGameAction::
-				quit (this, SLOT (close()), this);
+    a =				KGameStandardAction::
+				quit (this, &Kubrick::close, this);
     actionCollection()->addAction (a->objectName(), a);
     // NOTE: KXmlGuiWindow::close() calls Kubrick::queryClose(), our real "quit"
 
     // Move menu.
-    a =				KStandardGameAction::
-				undo (game, SLOT (undoMove()), this);
+    a =				KGameStandardAction::
+				undo (game, &Game::undoMove, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setToolTip		(i18n("Undo the last move."));
     a->setWhatsThis		(i18n("Undo the last move."));
 
-    a =				KStandardGameAction::
-				redo (game, SLOT (redoMove()), this);
+    a =				KGameStandardAction::
+				redo (game, &Game::redoMove, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setToolTip		(i18n("Redo a previously undone move."));
     a->setWhatsThis		(i18n("Redo a previously undone move "
 				"(repeatedly from the start if required)."));
 
-    a =				KStandardGameAction::
-				solve (game, SLOT (solveCube()), this);
+    a =				KGameStandardAction::
+				solve (game, &Game::solveCube, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setToolTip		(i18n("Show the solution of the puzzle."));
     a->setWhatsThis		(i18n("Show the solution of the puzzle by "
 				"undoing and re-doing all shuffling moves."));
 
-    a =				KStandardGameAction::
-				demo (game, SLOT (toggleDemo()), this);
+    a =				KGameStandardAction::
+				demo (game, &Game::toggleDemo, this);
     actionCollection()->addAction (a->objectName(), a);
     a->setText			(i18n("Main &Demo"));
     a->setToolTip		(i18n("Run a demonstration of puzzle moves."));
@@ -288,12 +285,12 @@ void Kubrick::initGUI()
 				"axes, thus making keyboard moves properly "
 				"meaningful."));
     a->setIcon			(QIcon::fromTheme( QStringLiteral( "go-home" )));
-    actionCollection()->setDefaultShortcut(a, Qt::Key_Home);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_Home);
     connect (a, &QAction::triggered, game, &Game::setStandardView);
 
     a = actionCollection()->addAction ( QStringLiteral( "redo_all" ));
     a->setText (i18n("Redo All"));
-    actionCollection()->setDefaultShortcut(a, Qt::SHIFT | Qt::Key_R);
+    KActionCollection::setDefaultShortcut(a, Qt::SHIFT | Qt::Key_R);
     connect (a, &QAction::triggered, game, &Game::redoAll);
 
     // Read-only display of Singmaster moves on the toolbar.
@@ -406,12 +403,12 @@ void Kubrick::initGUI()
     // Settings menu.
     b = new KToggleAction (i18n("&Watch Shuffling"), this);
     actionCollection()->addAction ( QStringLiteral( "watch_shuffling" ), b);
-    actionCollection()->setDefaultShortcut(b, Qt::Key_W);
+    KActionCollection::setDefaultShortcut(b, Qt::Key_W);
     connect(b, &KToggleAction::triggered, game, &Game::watchShuffling);
 
     b = new KToggleAction (i18n("Watch Your &Own Moves"), this);
     actionCollection()->addAction ( QStringLiteral( "watch_moves" ), b);
-    actionCollection()->setDefaultShortcut(b, Qt::Key_O);
+    KActionCollection::setDefaultShortcut(b, Qt::Key_O);
     connect(b, &KToggleAction::triggered, game, &Game::watchMoves);
 
     // DISCONTINUED a = actionCollection()->addAction ( QLatin1String( "enable_messages" ));
@@ -437,45 +434,43 @@ void Kubrick::initGUI()
     // Keys to choose the axis for a slice move (X, Y or Z).
     a = actionCollection()->addAction ( QStringLiteral( "x_axis" ));
     a->setText (i18n("X Axis"));
-    actionCollection()->setDefaultShortcut(a, Qt::Key_X);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_X);
     connect(a, &QAction::triggered, game, [this] { game->setMoveAxis(0); });
 
     a = actionCollection()->addAction ( QStringLiteral( "y_axis" ));
     a->setText (i18n("Y Axis"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_Y);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_Y);
     connect(a, &QAction::triggered, game, [this] { game->setMoveAxis(1); });
 
     a = actionCollection()->addAction ( QStringLiteral( "z_axis" ));
     a->setText (i18n("Z Axis"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_Z);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_Z);
     connect(a, &QAction::triggered, game, [this] { game->setMoveAxis(2); });
 
     // Keys to choose the slice number for a slice move.
-    char ident [10];
-    strcpy (ident, "slice n");
+    const QString ident = QStringLiteral("slice %1");
     for (int i = 1; i <= 6; i++) {
-        sprintf (ident, "slice %d", i);
-        a = actionCollection()->addAction (ident);
+        a = actionCollection()->addAction (ident.arg(i));
         a->setText (i18n("Slice %1", i));
-        actionCollection()->setDefaultShortcut(a, Qt::Key_0 + i);
+        KActionCollection::setDefaultShortcut(a, Qt::Key_0 + i);
         connect(a, &QAction::triggered, game, [this, i] { game->setMoveSlice(i); });
     }
 
     // Key to select a rotation of the whole cube (mapped as "slice 0").
     a = actionCollection()->addAction ( QStringLiteral( "turn_cube" ));
     a->setText (i18n("Turn whole cube"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_C);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_C);
     connect(a, &QAction::triggered, game, [this] { game->setMoveSlice(0); });
 
     // Keys to choose the direction for a slice move (clock or anti-clock).
     a = actionCollection()->addAction ( QStringLiteral( "anti_clockwise" ));
     a->setText (i18n("Anti-clockwise"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_Left);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_Left);
     connect (a, &QAction::triggered, game, [this] { game->setMoveDirection(0); });
 
     a = actionCollection()->addAction ( QStringLiteral( "clockwise" ));
     a->setText (i18n("Clockwise"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_Right);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_Right);
     connect (a, &QAction::triggered, game, [this] { game->setMoveDirection(1); });
 
     // Keys for Singmaster (sm) moves.
@@ -509,7 +504,7 @@ void Kubrick::initGUI()
     // IDW - Key for switching the background (temporary) - FIX IT FOR KDE 4.2.
     a = actionCollection()->addAction ( QStringLiteral( "switch_background" ));
     a->setText (i18n("Switch Background"));
-    actionCollection()->setDefaultShortcut(a,Qt::Key_K);
+    KActionCollection::setDefaultShortcut(a, Qt::Key_K);
     connect (a, &QAction::triggered, game, &Game::switchBackground);
 }
 
@@ -520,21 +515,21 @@ QAction * Kubrick::mapAction (const QString & name,
     QAction * a;
     a = actionCollection()->addAction (name);
     a->setText (text);
-    actionCollection()->setDefaultShortcut(a,key);
+    KActionCollection::setDefaultShortcut(a, key);
     connect (a, &QAction::triggered, game, [this, mapping] { game->smInput(mapping); });
     return a;
 }
 
 
-void Kubrick::setToggle (const char * actionName, bool onOff)
+void Kubrick::setToggle (const QString &actionName, bool onOff)
 {
-    ((KToggleAction *) ACTION (actionName))->setChecked (onOff);
+    ((KToggleAction *) actionCollection()->action (actionName))->setChecked (onOff);
 }
 
 
-void Kubrick::setAvail (const char * actionName, bool onOff)
+void Kubrick::setAvail (const QString &actionName, bool onOff)
 {
-    ((QAction *) ACTION (actionName))->setEnabled (onOff);
+    ((QAction *) actionCollection()->action (actionName))->setEnabled (onOff);
 }
 
 
@@ -563,10 +558,10 @@ int Kubrick::fillPuzzleList (KSelectAction * s, const PuzzleItem itemList [])
 
 
 void Kubrick::fillDemoList (const DemoItem itemList [], QList<QAction *> & list,
-				const char * uilist, void(Kubrick::*slot)())
+				const QString &uilist, void(Kubrick::*slot)())
 {
     // Generate an action list with one action for each item in the demo list.
-    for (uint i = 0; (strcmp (itemList[i].filename, "END") != 0); i++) {
+    for (uint i = 0; itemList[i].filename != QLatin1String("END"); i++) {
     QAction * t = new QAction (KLocalizedString(itemList[i].menuText).toString(), this);
     actionCollection()->addAction ( QStringLiteral("%1%2" ).arg(uilist).arg(i), t);
 	t->setData (i);		// Save the index of the item inside the action.
